@@ -353,6 +353,12 @@ class DateTime:
         except:
             raise SyntaxError('Unable to parse %s, %s' % (args, kw))
 
+    def __setstate__(self, state):
+        self.__dict__.clear()  # why doesn't Python's unpickler do this?
+        self.__dict__.update(state)
+        if '_micros' not in state:
+            self._micros = self._upgrade_old()
+
     def _parse_args(self, *args, **kw):
         """Return a new date-time object.
 
@@ -586,11 +592,8 @@ class DateTime:
                 DateTime instance.
                 """
                 t = arg.timeTime()
-                tz = arg.timezone()
-                ms = (t - math.floor(t))
                 s,d = _calcSD(t)
-                yr,mo,dy,hr,mn,sc = gmtime(t)[:6]
-                sc = sc + ms
+                yr,mo,dy,hr,mn,sc,tz = arg.parts()
             
             elif isinstance(arg, datetime):
                 yr,mo,dy,hr,mn,sc,numerictz,tznaive=self._parse_iso8601_preserving_tznaive(arg.isoformat())
@@ -599,7 +602,8 @@ class DateTime:
                     tz = None
                 else:
                     self._timezone_naive = False
-                    tz = arg.tzinfo.zone
+                    # if we have a pytz tzinfo, use the `zone` attribute as a key
+                    tz = getattr(arg.tzinfo, 'zone', numerictz)
                 ms = sc - math.floor(sc)
                 x = _calcDependentSecond2(yr,mo,dy,hr,mn,sc)
 
@@ -1850,5 +1854,5 @@ class strftimeFormatter:
 # Module methods
 def Timezones():
     """Return the list of recognized timezone names"""
-    return PytzCache._zlst
+    return sorted(list(PytzCache._zmap.values()))
 
