@@ -19,7 +19,7 @@ import unittest
 
 from DateTime.DateTime import _findLocalTimeZoneName, _cache
 from DateTime import DateTime
-from datetime import datetime
+from datetime import datetime, tzinfo, timedelta
 import pytz
 import legacy
 
@@ -34,6 +34,24 @@ else:
 DATADIR = os.path.dirname(os.path.abspath(f))
 del f
 
+ZERO = timedelta(0)
+
+class FixedOffset(tzinfo):
+    """Fixed offset in minutes east from UTC."""
+
+    def __init__(self, offset, name):
+        self.__offset = timedelta(minutes = offset)
+        self.__name = name
+
+    def utcoffset(self, dt):
+        return self.__offset
+
+    def tzname(self, dt):
+        return self.__name
+
+    def dst(self, dt):
+        return ZERO
+
 
 class DateTimeTests(unittest.TestCase):
 
@@ -193,14 +211,61 @@ class DateTimeTests(unittest.TestCase):
         self.failUnless(dt != dt1)
         self.failUnless(not (dt == dt1))
 
-    def testUpgradeOldInstances(self):
+    def test_compare_old_instances(self):
         # Compare dates that don't have the _micros attribute yet
+        # (e.g., from old pickles).
         dt = DateTime('1997/1/1')
         dt1 = DateTime('1997/2/2')
+        dt._millis = dt._micros / 1000
         del dt._micros
+        dt1._millis = dt1._micros / 1000
         del dt1._micros
         self.testCompareOperations(dt, dt1)
 
+    def test_compare_old_new_instances(self):
+        # Compare a date without _micros attribute (e.g., from an old
+        # pickle) with one that does.
+        dt = DateTime('1997/1/1')
+        dt1 = DateTime('1997/2/2')
+        dt._millis = dt._micros / 1000
+        del dt._micros
+        self.testCompareOperations(dt, dt1)
+
+    def test_compare_new_old_instances(self):
+        # Compare a date with _micros attribute with one that does not
+        # (e.g., from an old pickle).
+        dt = DateTime('1997/1/1')
+        dt1 = DateTime('1997/2/2')
+        dt1._millis = dt._micros / 1000
+        del dt1._micros
+        self.testCompareOperations(dt, dt1)
+
+    def test_strftime_old_instance(self):
+        # https://bugs.launchpad.net/zope2/+bug/290254
+        # Ensure that dates without _micros attribute (e.g., from old
+        # pickles) still render correctly in strftime.
+        ISO = '2001-10-10T00:00:00+02:00'
+        dt = DateTime(ISO)
+        dt._millis = dt._micros / 1000
+        del dt._micros
+        self.assertEqual(dt.strftime('%Y'), '2001')
+
+        # Now, create one via pickling / unpickling.
+        from cPickle import dumps, loads
+        self.assertEqual(loads(dumps(dt)).strftime('%Y'), '2001')
+
+    def test___setstate___without_micros(self):
+        ISO = '2001-10-10T00:00:00+02:00'
+        dt = DateTime(ISO)
+        micros = dt._micros
+        dt._millis = dt._micros / 1000
+        del dt._micros
+        state = dt.__dict__
+
+        dt1 = DateTime()
+        dt1.__setstate__(state)
+        self.assertEqual(dt1._micros, micros)
+
     def testTZ2(self):
         # Time zone manipulation test 2
         dt = DateTime()
@@ -383,8 +448,10 @@ class DateTimeTests(unittest.TestCase):
     def testCopyConstructor(self):
         d = DateTime('2004/04/04')
         self.assertEqual(DateTime(d), d)
-        d = DateTime('1999/04/12')
-        self.assertEqual(DateTime(d), d)
+        self.assertEqual(str(DateTime(d)), str(d))
+        d2 = DateTime('1999/04/12 01:00:00')
+        self.assertEqual(DateTime(d2), d2)
+        self.assertEqual(str(DateTime(d2)), str(d2))
 
     def testCopyConstructorPreservesTimezone(self):
         # test for https://bugs.launchpad.net/zope2/+bug/200007
@@ -395,8 +462,10 @@ class DateTimeTests(unittest.TestCase):
         self.assertEqual(DateTime(d).timezone(), d.timezone())
         d2 = DateTime('2008/04/25 12:00:00 EST')
         self.assertEqual(DateTime(d2).timezone(), d2.timezone())
+        self.assertEqual(str(DateTime(d2)), str(d2))
         d3 = DateTime('2008/04/25 12:00:00 PST')
         self.assertEqual(DateTime(d3).timezone(), d3.timezone())
+        self.assertEqual(str(DateTime(d3)), str(d3))
 
 
     def testRFC822(self):
@@ -561,8 +630,17 @@ class DateTimeTests(unittest.TestCase):
         real_failures = list(set(failures).difference(set(expected_failures)))
             
         self.failIf(real_failures, '\n'.join(real_failures))
-            
-            
+    
+    def testBasicTZ(self):
+        """psycopg2 supplies it's own tzinfo instances, with no `zone` attribute
+        """
+        tz = FixedOffset(60, 'GMT+1')
+        dt1 = datetime(2008, 8, 5, 12, 0, tzinfo=tz)
+        DT = DateTime(dt1)
+        dt2 = DT.asdatetime()
+        offset1 = dt1.tzinfo.utcoffset(dt1)
+        offset2 = dt2.tzinfo.utcoffset(dt2)
+        self.assertEqual(offset1, offset2)
 
 
 def test_suite():
